
# s21  simulator rewrite from C, 25 March 2020
# version 1.0

import array
import load as ld

from load import getM as getM     # getting and setting memory
from load import setM as setM


R = array.array('L',[0]*32)       # registers
PC = 0
savePC = 0
ninst = 0                         # number of executed inst
runflag = 1
MAXRUN = 100000                   # max simulation run

def signx2(x):          # sign x 22 bits
    if( x & 0x0200000 ):
        return x | 0xFFC00000
    return x

def setR(r1,d):
    R[r1] = d

def dojump(cond,ads):
    global PC
    if( cond != 0 ):
        PC = ads
    
def docall(r1,ads):
    global PC
    R[r1] = PC
    PC = ads

def dopush(r1,r2):
    R[r1] += 1
    setM(R[r1], R[r2])
    
def dopop(r1,r2):
    R[r2] = getM(R[r1])
    R[r1] -= 1

def trap(r1,n):
    global runflag
    
    if( n == 0 ):
        runflag = 0
        print("stop execute",ninst,"instructions")        
    elif( n == 1 ):
        print(R[r1],end="")
    elif( n == 2 ):
        print(chr(R[r1]),end="")
        

def doxop(op,r1,r2,r3):
    switcher = {
        0: lambda: setR(r1, R[r2] + R[r3]),         # add
        1: lambda: setR(r1, R[r2] - R[r3]),         # sub
        2: lambda: setR(r1, R[r2] * R[r3]),         # mul
        3: lambda: setR(r1, R[r2] / R[r3]),         # div
        4: lambda: setR(r1, R[r2] & R[r3]),         # and
        5: lambda: setR(r1, R[r2] | R[r3]),         # or
        6: lambda: setR(r1, R[r2] ^ R[r3]),         # xor
        7: lambda: setR(r1, R[r2] == R[r3]),        # eq
        8: lambda: setR(r1, R[r2] != R[r3]),        # ne
        9: lambda: setR(r1, R[r2] <  R[r3]),        # lt
        10: lambda: setR(r1, R[r2] <= R[r3]),       # le
        11: lambda: setR(r1, R[r2] >  R[r3]),       # gt
        12: lambda: setR(r1, R[r2] >= R[r3]),       # ge
        13: lambda: setR(r1, R[r2] << R[r3]),       # shl
        14: lambda: setR(r1, R[r2] >> R[r3]),       # shr
        15: lambda: setR(r1, R[r2] %  R[r3]),       # mod
        16: lambda: setR(r1, R[r2]),                # mov
        17: lambda: setR(r1, getM(R[r2]+R[r3])),    # ldx
        18: lambda: setM(R[r2]+R[r3], R[r1]),       # stx
        19: lambda: dojump(1, R[r1]),               # ret
        20: lambda: trap(r1, r2),                   # trap
        21: lambda: dopush(r1, r2),                 # push
        22: lambda: dopop(r1, r2),                  # pop
        23: lambda: setR(r1, R[r2] == 0),           # not
    }
    doxop = switcher.get(op, lambda: print("undefined op"))
    doxop()
    
def run():
    global PC, ninst,savePC
    
    savePC = PC
    IR = getM(PC)
    PC += 1
    ads = ld.IRads(IR)      # signx 17 bits
    d = ld.IRdisp(IR)
    r1 = ld.IRr1(IR)
    r2 = ld.IRr2(IR)
    r3 = ld.IRr3(IR)
    ninst += 1
    op = ld.IRop(IR)
    if( op == 31 ):
        doxop(ld.IRxop(IR),r1,r2,r3)
    else:
        switcher = {
            0: lambda: 0,                           # nop
            1: lambda: setR(r1, getM(ads)),         # ld ads
            2: lambda: setR(r1, getM(R[r2]+d)),     # ldd
            3: lambda: setM(ads, R[r1]),            # st ads
            4: lambda: setM(R[r2]+d, R[r1]),        # std
            5: lambda: setR(r1, signx2(ads)),       # mvi
            6: lambda: dojump(1,ads),               # jmp
            7: lambda: docall(r1,ads),              # jal
            8: lambda: dojump(R[r1] != 0,ads),      # jt
            9: lambda: dojump(R[r1] == 0,ads),      # jf
            10: lambda: setR(r1, R[r2] + d),        # addi
            11: lambda: setR(r1, R[r2] - d),        # subi
            12: lambda: setR(r1, R[r2] * d),        # muli
            13: lambda: setR(r1, R[r2] / d),        # divi
            14: lambda: setR(r1, R[r2] & d),        # andi
            15: lambda: setR(r1, R[r2] | d),        # ori
            16: lambda: setR(r1, R[r2] ^ d),        # xori
            17: lambda: setR(r1, R[r2] == d),       # eqi
            18: lambda: setR(r1, R[r2] != d),       # nei
            19: lambda: setR(r1, R[r2] <  d),       # lti
            20: lambda: setR(r1, R[r2] <= d),       # lei
            21: lambda: setR(r1, R[r2] >  d),       # gti
            22: lambda: setR(r1, R[r2] >= d),       # gei
            23: lambda: setR(r1, R[r2] << d),       # shli
            24: lambda: setR(r1, R[r2] >> d),       # shri
            25: lambda: setR(r1, R[r2] %  d),       # modi
        }
        doop = switcher.get(op, lambda: print("undefined op"))
        doop()

def show():  
    print("PC",savePC," ",end="")
    ld.disassemble(getM(savePC))
    print("  ",end="")
    for i in range(0,9):
        print(" r",i,":",R[i]," ",sep="",end="")
    print("\n  ",end="")
    for i in range(24,32):
        print(" r",i,":",R[i]," ",sep="",end="")
    print()
    
def main():  
    inp = input("object file:").strip().split()
    fname = inp[0]
    ld.loadprogram(fname)
#    ld.dodisassem()    
    interp()

# -------------- monitor -----------------

# mon (monitor) rewrite from C, 26 March 2020

breakpoint = 10000

def help():
    print("g - go")
    print("t - single step")
    print("d ads n - dump memory") 
    print("r - show registers")
    print("s [rn,mn,pc] v - set [reg,mem,pc]")
    print("b ads - set breakpoint")
    print("c - clear breakpoint")
    print("h - this help")
    print("q - quit")

def dumpreg():
    for i in range(0,32):
        if( i%8 == 0):
            print()
        print("  r",i,":",R[i],sep="",end="")

def dumpmem(ads,n):
    k = 0
    print(ads,":",end="")
    for i in range(ads, ads+n):
        print(getM(i)," ",end="")
        k += 1
        if( k%10 == 0 ):
            print("\n",i+1,":",end="")
    
regname = ['r0','r1','r2','r3','r4','r5','r6','r7','r8','r9',
           'r10','r11','r12','r13','r14','r15','r16','r17','r18',
           'r19','r20','r21','r22','r23','r24','r25','r26','r27',
           'r28','r29','r30','r31']

def setvalue(cmd):
    global PC

#    print(cmd)
    a = cmd[1]
    v = int(cmd[2])
    if( a in regname ):
        i = int(a[1:])
        R[i] = v
    elif( a[0] == 'm' ):
        setM(int(a[1:]),v)
    elif( a == "pc" ):
        PC = v
    else:
        print("set unknown")

def trace():
    global runflag
    
    while( (PC != breakpoint) and runflag ):
        if( ninst > MAXRUN ):
            print("infinite loop")
            runflag = 0
        else:
            run()
#            show()
            
def interp():
    global breakpoint
    
    runf = 1
    while(runf):
        cmd = input(">").split()
 #       print(cmd)
        c = cmd[0]
        if( c == 'g'):
            trace()
        elif( c == 't'):
            run()
            show()
        elif( c == 'r'):
            dumpreg()
            print()
        elif( c == 'd'):
            dumpmem(int(cmd[1]), int(cmd[2]))
            print()
        elif( c == 's'):
            setvalue(cmd)
        elif( c == 'b'):
            breakpoint = int(cmd[1])
            print("breakpoin at",breakpoint)
        elif( c == 'c'):
            breakpoint = 10000
        elif( c == 'h'):
            help()
        elif( c == 'q'):
            runf = 0
        else:
            print("unknown command")

# --------------------------------
            
main()
